
from assembly_payments.services.base import BaseService
from assembly_payments.types import Callback, CallbackRequest, CallbackUpdateRequest


class CallbackService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/callbacks"

    def get(self, callback_id):
        response = self._execute(CallbackService.GET, f"{self.endpoint}/{callback_id}")
        return Callback(**response)

    def list(self, *args, **kwargs):
        response = self._execute(CallbackService.GET, self.endpoint)
        return list(map(lambda x: Callback(**x), response.get("callbacks", [])))

    def create(self, **kwargs):
        data = CallbackRequest(**kwargs)
        response = self._execute(CallbackService.POST, self.endpoint, data=data.dict(exclude_none=True))
        return Callback(**response)

    def update(self, callback_id, **kwargs):
        data = CallbackUpdateRequest(**kwargs)
        response = self._execute(CallbackService.PATCH, f"{self.endpoint}/{callback_id}", data=data.dict(exclude_none=True))
        return Callback(**response['callbacks'])

    def delete(self, callback_id):
        self._execute(CallbackService.DELETE, f"{self.endpoint}/{callback_id}")




from assembly_payments.services.base import BaseService
from assembly_payments.types import BatchTransaction


class BatchTransactionService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/batch_transactions"

    def get(self, transaction_id):
        response = self._execute(
            BatchTransactionService.GET, f"{self.endpoint}/{transaction_id}"
        )
        return BatchTransaction(**response)

    def list(self):
        response = self._execute(BatchTransactionService.GET, f"{self.endpoint}")
        return list(map(lambda x: BatchTransaction(**x), response.get("batch_transactions", [])))


from assembly_payments.services.base import BaseService
from assembly_payments.types import Company, CompanyRequest


class CompanyService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/companies"

    def get(self, company_id):
        response = self._execute(CompanyService.GET, f"{self.endpoint}/{company_id}")
        return Company(**response["companies"])

    def list(self, *args, **kwargs):
        response = self._execute(CompanyService.GET, self.endpoint)
        return list(map(lambda x: Company(**x), response.get("companies", [])))

    def create(self, **kwargs):
        data = CompanyRequest(**kwargs)
        response = self._execute(CompanyService.POST, self.endpoint, data=data.dict())
        return Company(**response["companies"])

    def update(self, company_id, **kwargs):
        data = CompanyRequest(**kwargs)
        response = self._execute(
            CompanyService.PATCH,
            f"{self.endpoint}/{company_id}",
            data=data.dict(exclude_none=True),
        )
        return Company(**response["companies"])



from assembly_payments.services.base import BaseService
from assembly_payments.types import (
    BankAccount,
    User,
    BankAccountRequest,
    ProcessNppPaymentRequest,
    TransactionStatesRequest, UserRequest,
)


class TestingService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.endpoint = "/testing"

    def process_npp_payment(self, **kwargs):
        data = ProcessNppPaymentRequest(**kwargs)
        response = self._execute(
            TestingService.PATCH,
            f"{self.endpoint}/wallet_accounts/process_npp_payin",
            data=data.dict(),
        )
        return response

    def move_pending_batch_transactions_to_batched(self):
        response = self._execute(
            TestingService.GET, f"/batch_transactions/export_transactions"
        )
        return response

    def update_batched_transaction_state(self, id, state):
        data = TransactionStatesRequest(exported_ids=[id], state=state)
        response = self._execute(
            TestingService.PATCH, f"/batches/{id}/transaction_states", data=data.dict()
        )
        return response

    def verify_user(self, user_id):
        return self._execute(
            TestingService.PATCH, f"/users/{user_id}/identity_verified"
        )
